#!/bin/sh -eu

. install2-init-functions

# find block device name corresponding to given block device name
find_device()
{
	find /sys/devices/ -path "*/$1" |head -n1|
		sed -n 's|.*block/\([^/]\+\).*|\1|p'
}

# device mapper dependencies
dmsetup_deps()
{
	dmsetup deps -o devname "$1" | sed 's/.*://;s/(//g;s/)//g'
}

# exclude source devices
exclude= 
dev=
mpoint=
fstype=
opts=
dummy=
while read dev mpoint fstype opts dummy; do
	[ "$mpoint" = '/' -o "$mpoint" = "$image_dir" ] || continue
	[ -n "${dev%%/dev/*}" ] || exclude="$exclude $(find_device ${dev##*/})"
	[ "$mpoint" = "$image_dir" ] || continue
	# search for the device on which the mounted iso is located
	if dmsetup status "$dev" >/dev/null 2>&1; then
		device=
		for device in $(dmsetup_deps $dev); do
			exclude="$exclude $(find_device $device)"
		done
	fi
	if losetup "$dev" >/dev/null 2>&1; then
		device="$(readlink -f /dev/block/$(losetup -nO BACK-MAJ:MIN "$dev" |
			sed 's/^[[:blank:]]*//g'))"
		exclude="$exclude $(find_device ${device##*/})"
	fi
done </proc/mounts

# exclude ide cdrom devices
for n in /proc/ide/hd*/media; do
	[ -f "$n" ] || continue
	if [ "$(cat $n)" = cdrom ]; then
		exclude="$exclude $(echo "$n" |cut -d/ -f4)"
	fi
done

# exclude scsi cdrom devices
for n in /dev/sr[0-9]*; do
	[ -b "$n" ] || continue
	exclude="$exclude ${n##*/}"
done

# actually exclude
if [ -s /etc/evms.conf ]; then
	sed -i "0,/^sysfs_devices /{p;d}; \
		s/^\([[:space:]]*\)exclude = \[ \]/\1exclude = [$exclude loop* mmcblk*rpmb mmcblk*boot* mtdblock* ]/" \
		/etc/evms.conf ||:
fi

# seek calmer, poor ol' optical drive (if any)
for dev in $exclude; do
	echo 1024 > /sys/block/"$dev"/queue/read_ahead_kb 2>/dev/null ||:
done

# slow down mdraid resync while packages are installed
modprobe md_mod ||:
if [ -f /proc/sys/dev/raid/speed_limit_min ]; then
	cat /proc/sys/dev/raid/speed_limit_min >/proc/sys/dev/raid/speed_limit_max
fi
