#!/bin/sh -fu

a= . install2-init-functions
CONFIG="$destdir/etc/sysconfig/network"
CONFIG_SYSTEMD="$destdir/etc/hostname"
[ -f "$CONFIG" ] || exit 0
. "$CONFIG"

[ "$HOSTNAME" = 'localhost.localdomain' ] || exit 0

proc="$(sed -n 's;^model name[[:blank:]:]*\(.\+\);\1;p' < /proc/cpuinfo \
	| head -1 \
	| cut -d' ' -f 2-3 \
	| sed 's;(.*);;' \
	| tr ' _/' '---' \
	| tr '[:upper:]' '[:lower:]')"

mactail="$(ip link show \
	| sed -n 's;^.*link/ether[[:blank:]]\+\([[:xdigit:]:]\+\).*$;\1;p' \
	| head -1 \
	| cut -d: -f4-6 \
	| tr -d ":")"

[ -n "$mactail" -o -n "$proc" ] || exit 0

hostname="comp${proc:+-}$proc${mactail:+-}$mactail"

# 1. Remove all digits from beginning [RFC-952], relaxed in [RFC-1123, 2.1]
# 2. Remove all non-alphanumeric characters from hostname
#    (base64 generates [:alnum:],[+/=], only [:alnum:][-.] allowed)
# 3. Convert everything to lowercase.
PATTERN='s/^[0-9]*//g; s|[^[:alnum:]]||g; s/.*/\L&/g'

if [ ${#hostname} -gt 13 ]; then
	if command -v pwqgen >/dev/null; then
		hostname=$(pwqgen random=28 | sed -e "$PATTERN")
	else
		hostname=$(head -c 32 /dev/urandom | base64 | sed -e "$PATTERN" | cut -c1-13)
	fi
fi
subst "s/HOSTNAME=localhost.localdomain/HOSTNAME=$hostname/" "$CONFIG"
echo "$hostname" > "$CONFIG_SYSTEMD"

:
